@@ -13,13 +13,11 @@
 #define __must_check
 #endif
 
-#define uninitialized_var(x) x = x
-
-#ifndef __weak
+#ifndef _weak
 #ifndef __CYGWIN__
-#define __weak	__attribute__((weak))
+#define _weak	__attribute__((weak))
 #else
-#define __weak
+#define _weak
 #endif
 #endif
 
